/*
 *****************************************************
 *
 *  SaVi by Robert Thurman (thurman@geom.umn.edu) and
 *          Patrick Worfolk (worfolk@alum.mit.edu).
 *
 *  Copyright (c) 1997 by The Geometry Center.
 *  This file is part of SaVi.  SaVi is free software;
 *  you can redistribute it and/or modify it only under
 *  the terms given in the file COPYRIGHT which you should
 *  have received along with this file.  SaVi may be
 *  obtained from:
 *  http://savi.sourceforge.net/
 *  http://www.geom.uiuc.edu/locate/SaVi
 *
 *****************************************************
 *
 * globals.c
 *
 * declaration and initialization of global variables.
 *
 * $Id: globals.c,v 1.12 2005/02/12 15:52:33 lloydwood Exp $
 */

#include <limits.h>

#include "constants.h"
#include "globals.h"

/* Is program running as a Geomview module? */
int geomview_module = FALSE;
int geomview_logo = TRUE;

/* has texturemapping been turned on? */
unsigned int texture_flag = FALSE;

/* has dynamic texturemapping been selected? */
unsigned int geomview_dynamic_texture_flag = TRUE;
unsigned int geomview_texture_with_map = FALSE;

/* has large coverage map been selected? */
unsigned int coverage_large_map = FALSE;

/* debug flag to avoid recompiling. Off by default. */
int debug = FALSE;

/* assume X is available for use by fisheye */
int use_X = TRUE;

int coverage_projection = CYLINDRICAL;

unsigned char * cyl_foreground = NULL;
unsigned char * sin_foreground = NULL;
unsigned char * unp_foreground = NULL;

/* presumes default cylindrical projection */
int Longitude_Center_Line = 0;
int coverage_display_center_longitude = 0;

char * first_filename = NULL;

/* empty string */
char EMPTY_str[] = "";

/* default values for panels and initial motion */
unsigned int motion = FALSE;
unsigned int reset = FALSE;
unsigned int single_step = FALSE;

double direction = 1.0;
double delta_t = 60.0;
double coverage_angle = COVERAGE_ANGLE;
double tracks_interval = 0;	/* in hours */
unsigned int transforms_needed = 0;

/* global consts - to link to tcl vars */
int Image_Width = IMAGE_WIDTH;
int Image_Height = IMAGE_HEIGHT;
int Fisheye_Diameter = FISHEYE_DIAMETER;
